@@ -1,32 +1,10 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2020 Yury Gubich <blue@macaw.me>
+// SPDX-FileCopyrightText: 2020 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "ClientWorker.h"
 // Qt
@@ -34,29 +12,43 @@
 #include <QSettings>
 #include <QNetworkAccessManager>
 // QXmpp
+#include <QXmppAccountMigrationManager.h>
+#include <QXmppAuthenticationError.h>
+#include <QXmppBlockingManager.h>
 #include <QXmppCarbonManagerV2.h>
-#include <QXmppMamManager.h>
-#include <QXmppPubSubManager.h>
-#include <QXmppUtils.h>
+#include <QXmppDiscoveryManager.h>
+#include <QXmppEncryptedFileSharingProvider.h>
+#include <QXmppEntityTimeManager.h>
 #include <QXmppFileSharingManager.h>
 #include <QXmppHttpFileSharingProvider.h>
-#include <QXmppEncryptedFileSharingProvider.h>
 #include <QXmppHttpUploadManager.h>
-#include <QXmppUploadRequestManager.h>
+#include <QXmppMamManager.h>
+#include <QXmppMixManager.h>
+#include <QXmppMovedManager.h>
+#include <QXmppOmemoManager.h>
 #include <QXmppPubSubBaseItem.h>
+#include <QXmppPubSubManager.h>
+#include <QXmppRosterManager.h>
+#include <QXmppStreamError.h>
+#include <QXmppUploadRequestManager.h>
+#include <QXmppUtils.h>
+#include <QXmppVCardManager.h>
+#include <QXmppVersionManager.h>
+#include <QXmppSasl2UserAgent.h>
+#include <QXmppCredentials.h>
 // Kaidan
 #include "AccountManager.h"
+#include "AccountMigrationManager.h"
 #include "AtmManager.h"
 #include "AvatarFileStorage.h"
 #include "DiscoveryManager.h"
+#include "EncryptionController.h"
 #include "Enums.h"
 #include "FutureUtils.h"
 #include "Kaidan.h"
 #include "LogHandler.h"
-#include "MessageHandler.h"
-#include "MessageModel.h"
-#include "OmemoCache.h"
-#include "OmemoManager.h"
+#include "MessageController.h"
+#include "OmemoDb.h"
 #include "PresenceCache.h"
 #include "RegistrationManager.h"
 #include "RosterManager.h"
@@ -72,37 +64,47 @@ ClientWorker::Caches::Caches(QObject *parent)
 	: settings(new Settings(parent)),
 	  vCardCache(new VCardCache(parent)),
 	  accountManager(new AccountManager(settings, vCardCache, parent)),
-	  msgModel(new MessageModel(parent)),
+	  presenceCache(new PresenceCache(parent)),
 	  rosterModel(new RosterModel(parent)),
-	  omemoCache(new OmemoCache(parent)),
 	  avatarStorage(new AvatarFileStorage(parent)),
-	  serverFeaturesCache(new ServerFeaturesCache(parent)),
-	  presenceCache(new PresenceCache(parent))
+	  serverFeaturesCache(new ServerFeaturesCache(parent))
 {
 }
 
 ClientWorker::ClientWorker(Caches *caches, Database *database, bool enableLogging, QObject* parent)
 	: QObject(parent),
 	  m_caches(caches),
-	  m_client(new QXmppClient(this)),
+	  m_client(new QXmppClient(QXmppClient::NoExtensions, this)),
 	  m_logger(new LogHandler(m_client, enableLogging, this)),
 	  m_enableLogging(enableLogging),
-	  m_networkManager(new QNetworkAccessManager(this))
+	  m_networkManager(new QNetworkAccessManager(this)),
+	  m_omemoDb(new OmemoDb(database, this, {}, this))
 {
+	m_atmManager = new AtmManager(m_client, database, this);
+
+	m_client->addNewExtension<QXmppAccountMigrationManager>();
+	m_client->addNewExtension<QXmppBlockingManager>();
 	m_client->addNewExtension<QXmppCarbonManagerV2>();
-	m_client->addExtension(new QXmppMamManager);
-	m_client->addExtension(new QXmppPubSubManager);
-	m_client->addNewExtension<QXmppUploadRequestManager>();
+	m_client->addNewExtension<QXmppDiscoveryManager>();
+	m_client->addNewExtension<QXmppEntityTimeManager>();
 	auto *uploadManager = m_client->addNewExtension<QXmppHttpUploadManager>(m_networkManager);
+	m_client->addNewExtension<QXmppMamManager>();
+	m_client->addNewExtension<QXmppPubSubManager>();
+	m_client->addNewExtension<QXmppMovedManager>();
+	m_client->setEncryptionExtension(m_client->addNewExtension<QXmppOmemoManager>(m_omemoDb));
+	m_client->addNewExtension<QXmppMessageReceiptManager>();
+	m_client->addNewExtension<QXmppRosterManager>(m_client);
+	m_client->addNewExtension<QXmppUploadRequestManager>();
+	m_client->addNewExtension<QXmppVCardManager>();
+	m_client->addNewExtension<QXmppVersionManager>();
+	m_client->addNewExtension<QXmppMixManager>();
 
 	m_registrationManager = new RegistrationManager(this, m_client, this);
 	m_vCardManager = new VCardManager(this, m_client, m_caches->avatarStorage, this);
 	m_rosterManager = new RosterManager(this, m_client, this);
-	m_messageHandler = new MessageHandler(this, m_client, this);
-	m_atmManager = new AtmManager(m_client, database, this);
-	m_omemoManager = new OmemoManager(m_client, database, this);
 	m_discoveryManager = new DiscoveryManager(m_client, this);
 	m_versionManager = new VersionManager(m_client, this);
+	m_accountMigrationManager = new AccountMigrationManager(this, this);
 
 	// file sharing manager
 	m_fileSharingManager = m_client->addNewExtension<QXmppFileSharingManager>();
@@ -116,10 +118,13 @@ ClientWorker::ClientWorker(Caches *caches, Database *database, bool enableLoggin
 	connect(m_client, &QXmppClient::disconnected, this, &ClientWorker::onDisconnected);
 
 	connect(m_client, &QXmppClient::stateChanged, this, &ClientWorker::onConnectionStateChanged);
-	connect(m_client, &QXmppClient::error, this, &ClientWorker::onConnectionError);
+	connect(m_client, &QXmppClient::errorOccurred, this, &ClientWorker::onConnectionError);
+
+	connect(m_client, &QXmppClient::credentialsChanged, this, [this]() {
+		m_caches->settings->setAuthCredentials(m_client->configuration().credentials());
+	});
 
 	connect(Kaidan::instance(), &Kaidan::logInRequested, this, &ClientWorker::logIn);
-	connect(Kaidan::instance(), &Kaidan::registrationFormRequested, this, &ClientWorker::connectToRegister);
 	connect(Kaidan::instance(), &Kaidan::logOutRequested, this, &ClientWorker::logOut);
 
 	// presence
@@ -135,10 +140,6 @@ ClientWorker::ClientWorker(Caches *caches, Database *database, bool enableLoggin
 			m_client->setActive(false);
 		}
 	});
-
-	// account deletion
-	connect(Kaidan::instance(), &Kaidan::deleteAccountFromClient, this, &ClientWorker::deleteAccountFromClient);
-	connect(Kaidan::instance(), &Kaidan::deleteAccountFromClientAndServer, this, &ClientWorker::deleteAccountFromClientAndServer);
 }
 
 void ClientWorker::startTask(const std::function<void ()> &task)
@@ -172,30 +173,34 @@ void ClientWorker::logIn()
 {
 	const auto jid = AccountManager::instance()->jid();
 	m_atmManager->setAccountJid(jid);
-	m_omemoManager->setAccountJid(jid);
+	m_omemoDb->setAccountJid(jid);
 
-	await(m_omemoManager->load(), this, [this]() {
-		if (!m_isFirstLoginAfterStart || m_caches->settings->authOnline()) {
-			// Store the latest online state which is restored when opening Kaidan again after closing.
-			m_caches->settings->setAuthOnline(true);
+	await(
+		EncryptionController::instance(),
+		[]() {
+			return EncryptionController::instance()->load();
+		},
+		this,
+		[this] () {
+			if (!m_isFirstLoginAfterStart || m_caches->settings->authOnline()) {
+				// Store the latest online state which is restored when opening Kaidan again after closing.
+				m_caches->settings->setAuthOnline(true);
 
-			QXmppConfiguration config;
-			config.setResource(AccountManager::instance()->jidResource());
-			config.setPassword(AccountManager::instance()->password());
-			config.setAutoAcceptSubscriptions(false);
+				QXmppConfiguration config;
+				config.setResource(AccountManager::instance()->jidResource());
+				config.setCredentials(m_caches->settings->authCredentials());
+				config.setPassword(AccountManager::instance()->password());
+				config.setAutoAcceptSubscriptions(false);
 
-			connectToServer(config);
+				// Servers using LDAP do not support SASL PLAIN authentication.
+				config.setDisabledSaslMechanisms({});
+
+				connectToServer(config);
+			}
+
+			m_isFirstLoginAfterStart = false;
 		}
-
-		m_isFirstLoginAfterStart = false;
-	});
-}
-
-void ClientWorker::connectToRegister()
-{
-	AccountManager::instance()->setHasNewCredentials(true);
-	m_registrationManager->setRegisterOnConnectEnabled(true);
-	connectToServer();
+	);
 }
 
 void ClientWorker::connectToServer(QXmppConfiguration config)
@@ -212,19 +217,24 @@ void ClientWorker::connectToServer(QXmppConfiguration config)
 		break;
 	case QXmppClient::DisconnectedState:
 		config.setJid(AccountManager::instance()->jid());
-		config.setStreamSecurityMode(QXmppConfiguration::TLSRequired);
+		config.setStreamSecurityMode(m_caches->settings->authTlsRequirement());
+		config.setIgnoreSslErrors(m_caches->settings->authTlsErrorsIgnored());
+		config.setResourcePrefix(m_caches->settings->authJidResourcePrefix());
+		config.setSasl2UserAgent(m_caches->accountManager->userAgent());
 
 		auto host = AccountManager::instance()->host();
-		if (!host.isEmpty())
+		if (!host.isEmpty()) {
 			config.setHost(host);
+		}
 
 		auto port = AccountManager::instance()->port();
 		if (port != PORT_AUTODETECT) {
 			config.setPort(port);
 
 			// Set the JID's domain part as the host if no custom host is set.
-			if (host.isEmpty())
+			if (host.isEmpty()) {
 				config.setHost(QXmppUtils::jidToDomain(AccountManager::instance()->jid()));
+			}
 		}
 
 		// Disable the automatic reconnection in case this connection attempt is not
@@ -245,15 +255,23 @@ void ClientWorker::connectToServer(QXmppConfiguration config)
 void ClientWorker::logOut(bool isApplicationBeingClosed)
 {
 	// Store the latest online state which is restored when opening Kaidan again after closing.
-	if (!isApplicationBeingClosed)
+	if (!isApplicationBeingClosed) {
 		m_caches->settings->setAuthOnline(false);
+	}
 
 	switch (m_client->state()) {
 	case QXmppClient::DisconnectedState:
 		break;
 	case QXmppClient::ConnectingState:
-		qDebug() << "[main] Tried to disconnect even if still connecting! Waiting for connecting to succeed and disconnect afterwards.";
-		m_isDisconnecting = true;
+		// Abort an ongoing registration if the application is being closed.
+		// Otherwise, wait for the client to connect in order to disconnect appropriately.
+		if (isApplicationBeingClosed && m_registrationManager->registerOnConnectEnabled()) {
+			m_client->disconnectFromServer();
+		} else {
+			qDebug() << "[main] Tried to disconnect even if still connecting! Waiting for connecting to succeed and disconnect afterwards.";
+			m_isDisconnecting = true;
+		}
+
 		break;
 	case QXmppClient::ConnectedState:
 		if (AccountManager::instance()->hasNewCredentials()) {
@@ -264,110 +282,92 @@ void ClientWorker::logOut(bool isApplicationBeingClosed)
 			AccountManager::instance()->setHasNewConnectionSettings(false);
 		}
 
-		await(m_omemoManager->unsubscribeFromDeviceLists(), this, [this] {
-			m_client->disconnectFromServer();
-		});
+		await(
+			EncryptionController::instance(),
+			[]() {
+				return EncryptionController::instance()->unload();
+			},
+			this,
+			[this]() {
+				m_client->disconnectFromServer();
+			}
+		);
 	}
 }
 
-void ClientWorker::deleteAccountFromClientAndServer()
-{
-	m_isAccountToBeDeletedFromClientAndServer = true;
-
-	// If the client is already connected, delete the account directly from the server.
-	// Otherwise, connect first and delete the account afterwards.
-	if (m_client->isAuthenticated()) {
-		m_registrationManager->deleteAccount();
-	} else {
-		m_isClientConnectedBeforeAccountDeletionFromServer = false;
-		logIn();
-	}
-}
-
-void ClientWorker::deleteAccountFromClient()
-{
-	// If the client is already disconnected, delete the account directly from the client.
-	// Otherwise, disconnect first and delete the account afterwards.
-	if (!m_client->isAuthenticated()) {
-		AccountManager::instance()->removeAccount(m_client->configuration().jidBare());
-		m_isAccountToBeDeletedFromClient = false;
-	} else {
-		m_omemoManager->resetOwnDevice().then(this, [this](auto &&) {
-			m_isAccountToBeDeletedFromClient = true;
-			logOut();
-		});
-	}
-}
-
-void ClientWorker::handleAccountDeletedFromServer()
-{
-	m_isAccountDeletedFromServer = true;
-}
-
-void ClientWorker::handleAccountDeletionFromServerFailed(const QXmppStanza::Error &error)
-{
-	emit Kaidan::instance()->passiveNotificationRequested(tr("Your account could not be deleted from the server. Therefore, it was also not removed from this app: %1").arg(error.text()));
-
-	m_isAccountToBeDeletedFromClientAndServer = false;
-
-	if (!m_isClientConnectedBeforeAccountDeletionFromServer)
-		logOut();
-}
-
 void ClientWorker::onConnected()
 {
 	// no mutex needed, because this is called from updateClient()
 	qDebug() << "[client] Connected successfully to server";
 
 	// If there was an error before, notify about its absence.
-	emit connectionErrorChanged(ClientWorker::NoError);
+	Q_EMIT connectionErrorChanged(ClientWorker::NoError);
 
-	// If the account could not be deleted from the server because the client was
-	// disconnected, delete it now.
-	if (m_isAccountToBeDeletedFromClientAndServer) {
-		m_registrationManager->deleteAccount();
-		return;
-	}
+	runOnThread(
+		AccountManager::instance(),
+		[]() {
+			return AccountManager::instance()->handleConnected();
+		},
+		this,
+		[this](bool handled) {
+			if (handled) {
+				return;
+			}
 
-	// Try to complete pending tasks which could not be completed while the client was
-	// offline and skip further normal tasks if at least one was completed after a login
-	// with old credentials.
-	if (startPendingTasks())
-		return;
+			// Try to complete pending tasks which could not be completed while the client was
+			// offline and skip further normal tasks if at least one was completed after a login
+			// with old credentials.
+			if (startPendingTasks())
+				return;
 
-	// If the client was connecting during a disconnection attempt, disconnect it now.
-	if (m_isDisconnecting) {
-		m_isDisconnecting = false;
-		logOut();
-		return;
-	}
+			// If the client was connecting during a disconnection attempt, disconnect it now.
+			if (m_isDisconnecting) {
+				m_isDisconnecting = false;
+				logOut();
+				return;
+			}
 
-	// The following tasks are only done after a login with new credentials or connection settings.
-	if (AccountManager::instance()->hasNewCredentials() || AccountManager::instance()->hasNewConnectionSettings()) {
-		if (AccountManager::instance()->hasNewCredentials()) {
-			emit loggedInWithNewCredentials();
+			// The following tasks are only done after a login with new credentials or connection
+			// settings.
+			if (AccountManager::instance()->hasNewCredentials() || AccountManager::instance()->hasNewConnectionSettings()) {
+				if (AccountManager::instance()->hasNewCredentials()) {
+					Q_EMIT loggedInWithNewCredentials();
+				}
+
+				// Store the valid connection data.
+				AccountManager::instance()->storeConnectionData();
+			}
+
+			// Enable auto reconnection so that the client is always trying to reconnect
+			// automatically in case of a connection outage.
+			m_client->configuration().setAutoReconnectionEnabled(true);
+
+			// Send message that could not be sent yet because the client was offline.
+			runOnThread(MessageController::instance(), [jid = AccountManager::instance()->jid()]() {
+				MessageController::instance()->sendPendingMessages();
+			});
+
+			// Send read markers that could not be sent yet because the client was offline.
+			runOnThread(RosterModel::instance(), [jid = AccountManager::instance()->jid()]() {
+				RosterModel::instance()->sendPendingReadMarkers(jid);
+			});
+
+			// Send message reactions that could not be sent yet because the client was offline.
+			runOnThread(MessageController::instance(), [jid = AccountManager::instance()->jid()]() {
+				MessageController::instance()->sendPendingMessageReactions(jid);
+			});
+
+			runOnThread(EncryptionController::instance(), []() {
+				EncryptionController::instance()->setUp();
+			});
 		}
-
-		// Store the valid settings.
-		AccountManager::instance()->storeConnectionData();
-	}
-
-	// Enable auto reconnection so that the client is always trying to reconnect
-	// automatically in case of a connection outage.
-	m_client->configuration().setAutoReconnectionEnabled(true);
-
-	MessageModel::instance()->sendPendingMessages();
-
-	// Send read markers that could not be sent yet because the client was offline.
-	runOnThread(RosterModel::instance(), [jid = AccountManager::instance()->jid()]() {
-		RosterModel::instance()->sendPendingReadMarkers(jid);
-	});
-
-	m_omemoManager->setUp();
+	);
 }
 
 void ClientWorker::onDisconnected()
 {
+	qDebug() << "[client] Disconnected";
+
 	if (m_isReconnecting) {
 		m_isReconnecting = false;
 		connectToServer(m_configToBeUsedOnNextConnect);
@@ -375,68 +375,70 @@ void ClientWorker::onDisconnected()
 		return;
 	}
 
-	// Delete the account from the client if the client was connected and had to
-	// disconnect first or if the account was deleted from the server.
-	if (m_isAccountToBeDeletedFromClient || (m_isAccountToBeDeletedFromClientAndServer && m_isAccountDeletedFromServer)) {
-		m_isAccountToBeDeletedFromClientAndServer = false;
-		m_isAccountDeletedFromServer = false;
-
-		deleteAccountFromClient();
-	}
+	runOnThread(AccountManager::instance(), []() {
+		AccountManager::instance()->handleDisconnected();
+	});
 }
 
 void ClientWorker::onConnectionStateChanged(QXmppClient::State connectionState)
 {
-	emit connectionStateChanged(Enums::ConnectionState(connectionState));
+	Q_EMIT connectionStateChanged(Enums::ConnectionState(connectionState));
 }
 
-void ClientWorker::onConnectionError(QXmppClient::Error error)
+void ClientWorker::onConnectionError(const QXmppError &error)
 {
 	// no mutex needed, because this is called from updateClient()
-	qDebug() << "[client] Disconnected:" << error;
+	qDebug() << "[client] Connection error:" << error.description;
 
-	switch (error) {
-	case QXmppClient::NoError:
-		Q_UNREACHABLE();
-	case QXmppClient::KeepAliveError:
-		emit connectionErrorChanged(ClientWorker::KeepAliveError);
-		break;
-	case QXmppClient::XmppStreamError: {
-		QXmppStanza::Error::Condition xmppStreamError = m_client->xmppStreamError();
-		qDebug() << "[client] XMPP stream error:" << xmppStreamError;
-		if (xmppStreamError == QXmppStanza::Error::NotAuthorized) {
-			emit connectionErrorChanged(ClientWorker::AuthenticationFailed);
-		} else {
-			emit connectionErrorChanged(ClientWorker::NotConnected);
-		}
-		break;
-	}
-	case QXmppClient::SocketError: {
-		QAbstractSocket::SocketError socketError = m_client->socketError();
-		switch (socketError) {
+	if (const auto socketError = error.value<QAbstractSocket::SocketError>()) {
+		switch (*socketError) {
 		case QAbstractSocket::ConnectionRefusedError:
 		case QAbstractSocket::RemoteHostClosedError:
-			emit connectionErrorChanged(ClientWorker::ConnectionRefused);
+			Q_EMIT connectionErrorChanged(ClientWorker::ConnectionRefused);
 			break;
 		case QAbstractSocket::HostNotFoundError:
-			emit connectionErrorChanged(ClientWorker::DnsError);
+			Q_EMIT connectionErrorChanged(ClientWorker::DnsError);
 			break;
 		case QAbstractSocket::SocketAccessError:
-			emit connectionErrorChanged(ClientWorker::NoNetworkPermission);
+			Q_EMIT connectionErrorChanged(ClientWorker::NoNetworkPermission);
 			break;
 		case QAbstractSocket::SocketTimeoutError:
-			emit connectionErrorChanged(ClientWorker::KeepAliveError);
+			Q_EMIT connectionErrorChanged(ClientWorker::KeepAliveError);
 			break;
 		case QAbstractSocket::SslHandshakeFailedError:
 		case QAbstractSocket::SslInternalError:
-			emit connectionErrorChanged(ClientWorker::TlsFailed);
+			Q_EMIT connectionErrorChanged(ClientWorker::TlsFailed);
 			break;
 		default:
-			emit connectionErrorChanged(ClientWorker::NotConnected);
+			Q_EMIT connectionErrorChanged(ClientWorker::NotConnected);
+		}
+		return;
+	} else if (error.holdsType<QXmpp::TimeoutError>()) {
+		Q_EMIT connectionErrorChanged(ClientWorker::KeepAliveError);
+		return;
+	} else if (error.holdsType<QXmpp::StreamError>()) {
+		Q_EMIT connectionErrorChanged(ClientWorker::NotConnected);
+		return;
+	} else if (const auto authenticationError = error.value<QXmpp::AuthenticationError>()) {
+		const auto type = authenticationError->type;
+		const auto text = authenticationError->text;
+
+		// Some servers require new users to confirm their account creation via instructions within
+		// an email message sent by the server to the user after a successful registration.
+		// That usually involves opening a URL.
+		// Afterwards, the account is activated and the user can log in.
+		if ((type == QXmpp::AuthenticationError::AccountDisabled ||
+			 type == QXmpp::AuthenticationError::NotAuthorized) &&
+			text.contains("activat") && text.contains("mail")) {
+			Q_EMIT connectionErrorChanged(ClientWorker::EmailConfirmationRequired);
+			return;
+		} else if (type == QXmpp::AuthenticationError::NotAuthorized) {
+			Q_EMIT connectionErrorChanged(ClientWorker::AuthenticationFailed);
+			return;
 		}
-		break;
-	}
 	}
+
+	Q_EMIT connectionErrorChanged(ClientWorker::UnknownError);
 }
 
 bool ClientWorker::startPendingTasks()
