@@ -1,32 +1,9 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2020 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
@@ -38,15 +15,13 @@
 #include "Enums.h"
 
 class AccountManager;
+class AccountMigrationManager;
 class AtmManager;
 class AvatarFileStorage;
 class Database;
 class DiscoveryManager;
 class LogHandler;
-class MessageHandler;
-class MessageModel;
-class OmemoCache;
-class OmemoManager;
+class OmemoDb;
 class RegistrationManager;
 class RosterManager;
 class RosterModel;
@@ -67,14 +42,13 @@ class PresenceCache;
 class ClientWorker : public QObject
 {
 	Q_OBJECT
+	Q_PROPERTY(AccountMigrationManager* accountMigrationManager READ accountMigrationManager CONSTANT)
 	Q_PROPERTY(RegistrationManager* registrationManager READ registrationManager CONSTANT)
 	Q_PROPERTY(VCardManager* vCardManager READ vCardManager CONSTANT)
 	Q_PROPERTY(RosterManager* rosterManager READ rosterManager CONSTANT)
-	Q_PROPERTY(MessageHandler* messageHandler READ messageHandler CONSTANT)
 	Q_PROPERTY(DiscoveryManager* discoveryManager READ discoveryManager CONSTANT)
 	Q_PROPERTY(VersionManager* versionManager READ versionManager CONSTANT)
 	Q_PROPERTY(AtmManager* atmManager READ atmManager CONSTANT)
-	Q_PROPERTY(OmemoManager* omemoManager READ omemoManager CONSTANT)
 
 public:
 	/**
@@ -91,7 +65,10 @@ public:
 		NoSupportedAuth,
 		KeepAliveError,
 		NoNetworkPermission,
-		RegistrationUnsupported
+		RegistrationUnsupported,
+		// The server requires the client to confirm an email message that the server sent to it.
+		EmailConfirmationRequired,
+		UnknownError,
 	};
 	Q_ENUM(ConnectionError)
 
@@ -101,12 +78,10 @@ public:
 		Settings *settings;
 		VCardCache *vCardCache;
 		AccountManager *accountManager;
-		MessageModel *msgModel;
+		PresenceCache *presenceCache;
 		RosterModel *rosterModel;
-		OmemoCache *omemoCache;
 		AvatarFileStorage *avatarStorage;
 		ServerFeaturesCache *serverFeaturesCache;
-		PresenceCache *presenceCache;
 	};
 
 	/**
@@ -121,6 +96,11 @@ public:
 		return m_registrationManager;
 	}
 
+	AccountMigrationManager *accountMigrationManager() const
+	{
+		return m_accountMigrationManager;
+	}
+
 	VCardManager *vCardManager() const
 	{
 		return m_vCardManager;
@@ -131,11 +111,6 @@ public:
 		return m_rosterManager;
 	}
 
-	MessageHandler *messageHandler() const
-	{
-		return m_messageHandler;
-	}
-
 	DiscoveryManager *discoveryManager() const
 	{
 		return m_discoveryManager;
@@ -151,11 +126,6 @@ public:
 		return m_atmManager;
 	}
 
-	OmemoManager *omemoManager() const
-	{
-		return m_omemoManager;
-	}
-
 	Caches *caches() const
 	{
 		return m_caches;
@@ -209,17 +179,11 @@ public:
 	 */
 	void finishTask();
 
-public slots:
 	/**
 	 * Connects to the server and logs in with all needed configuration variables.
 	 */
 	void logIn();
 
-	/**
-	 * Connects to the server and requests a data form for account registration.
-	 */
-	void connectToRegister();
-
 	/**
 	 * Connects to the server with a minimal configuration.
 	 *
@@ -237,52 +201,29 @@ public slots:
 	 */
 	void logOut(bool isApplicationBeingClosed = false);
 
-	/**
-	 * Deletes the account data from the client and server.
-	 */
-	void deleteAccountFromClientAndServer();
-
-	/**
-	 * Deletes the account data from the configuration file and database.
-	 */
-	void deleteAccountFromClient();
-
-	/**
-	 * Called when the account is deleted from the server.
-	 */
-	void handleAccountDeletedFromServer();
-
-	/**
-	 * Called when the account could not be deleted from the server.
-	 *
-	 * @param error error of the failed account deletion
-	 */
-	void handleAccountDeletionFromServerFailed(const QXmppStanza::Error &error);
-
-signals:
 	/**
 	 * Emitted when an authenticated connection to the server is established with new
 	 * credentials for the first time.
 	 *
 	 * The client will be in connected state when this is emitted.
 	 */
-	void loggedInWithNewCredentials();
+	Q_SIGNAL void loggedInWithNewCredentials();
 
 	/**
 	 * Emitted when the client's connection state changed.
 	 *
 	 * @param connectionState new connection state
 	 */
-	void connectionStateChanged(Enums::ConnectionState connectionState);
+	Q_SIGNAL void connectionStateChanged(Enums::ConnectionState connectionState);
 
 	/**
 	 * Emitted when the client failed to connect to the server.
 	 *
 	 * @param error new connection error
 	 */
-	void connectionErrorChanged(ClientWorker::ConnectionError error);
+	Q_SIGNAL void connectionErrorChanged(ClientWorker::ConnectionError error);
 
-private slots:
+private:
 	/**
 	 * Called when an authenticated connection to the server is established.
 	 */
@@ -305,9 +246,8 @@ private slots:
 	 *
 	 * @param error new connection error
 	 */
-	void onConnectionError(QXmppClient::Error error);
+	void onConnectionError(const QXmppError &error);
 
-private:
 	/**
 	 * Starts a pending (enqueued) task (e.g. a password change) if the variable (e.g. a
 	 * password) could not be changed on the server before because the client was not
@@ -325,14 +265,15 @@ private:
 	QNetworkAccessManager *m_networkManager;
 
 	RegistrationManager *m_registrationManager;
+	AccountMigrationManager *m_accountMigrationManager;
 	VCardManager *m_vCardManager;
 	RosterManager *m_rosterManager;
 	AtmManager *m_atmManager;
-	OmemoManager *m_omemoManager;
-	MessageHandler *m_messageHandler;
 	DiscoveryManager *m_discoveryManager;
 	VersionManager *m_versionManager;
 
+	OmemoDb *m_omemoDb;
+
 	QXmppFileSharingManager *m_fileSharingManager;
 	std::shared_ptr<QXmppHttpFileSharingProvider> m_httpProvider;
 	std::shared_ptr<QXmppEncryptedFileSharingProvider> m_encryptedProvider;
@@ -343,10 +284,4 @@ private:
 	bool m_isReconnecting = false;
 	bool m_isDisconnecting = false;
 	QXmppConfiguration m_configToBeUsedOnNextConnect;
-
-	// These variables are used for checking the state of an ongoing account deletion.
-	bool m_isAccountToBeDeletedFromClient = false;
-	bool m_isAccountToBeDeletedFromClientAndServer = false;
-	bool m_isAccountDeletedFromServer = false;
-	bool m_isClientConnectedBeforeAccountDeletionFromServer = true;
 };
