@@ -1,32 +1,8 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2022 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2022 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "FileSharingController.h"
 
@@ -36,7 +12,6 @@
 #include <QDir>
 #include <QFile>
 #include <QMimeDatabase>
-#include <QRandomGenerator>
 #include <QImage>
 #include <QStandardPaths>
 #include <QStringBuilder>
@@ -53,14 +28,17 @@
 #include <QXmppBitsOfBinaryDataList.h>
 #include <QXmppOutOfBandUrl.h>
 #include <QXmppUploadRequestManager.h>
+#include <QXmppPromise.h>
 
 #include <KFileUtils>
 
+#include "Account.h"
+#include "AccountDb.h"
+#include "Globals.h"
 #include "Kaidan.h"
 #include "FutureUtils.h"
 #include "FileProgressCache.h"
 #include "MessageDb.h"
-#include "MessageHandler.h"
 #include "Algorithms.h"
 #include "ServerFeaturesCache.h"
 
@@ -76,11 +54,6 @@ auto find(T &container, Value value)
 	return std::find(container.begin(), container.end(), std::forward<Value>(value));
 }
 
-static qint64 generateFileId()
-{
-	return QRandomGenerator::system()->generate64();
-}
-
 // ported from https://github.com/SergioBenitez/Rocket/blob/2cee4b459492136d616e5863c54754b135e41572/core/lib/src/fs/file_name.rs#L112
 ///
 /// Removes problematic (as in reserved characters, filenames with special meaning etc.)
@@ -92,19 +65,19 @@ static std::optional<std::pair<QString, QString>> sanitizeFilename(QStringView f
 	constexpr std::array bad_chars = {
 #ifdef Q_OS_UNIX
 		// These have special meaning in a file name.
-		'.', '/', '\\',
+		QLatin1Char('.'), QLatin1Char('/'), QLatin1Char('\\'),
 
 		// These are treated specially by shells.
-		'<', '>', '|', ':', '(', ')', '&', ';', '#', '?', '*',
+		QLatin1Char('<'), QLatin1Char('>'), QLatin1Char('|'), QLatin1Char(':'), QLatin1Char('('), QLatin1Char(')'), QLatin1Char('&'), QLatin1Char(';'), QLatin1Char('#'), QLatin1Char('?'), QLatin1Char('*'),
 #else
 		// Microsoft says these are invalid.
-		'.', '<', '>', ':', '"', '/', '\\', '|', '?', '*',
+		QLatin1Char('.'), QLatin1Char('<'), QLatin1Char('>'), QLatin1Char(':'), QLatin1Char('"'), QLatin1Char('/'), QLatin1Char('\\'), QLatin1Char('|'), QLatin1Char('?'), QLatin1Char('*'),
 
 		// `cmd.exe` treats these specially.
-		',', ';', '=',
+		QLatin1Char(','), QLatin1Char(';'), QLatin1Char('='),
 
 		// These are treated specially by unix-like shells.
-		'(', ')', '&', '#',
+		QLatin1Char('('), QLatin1Char(')'), QLatin1Char('&'), QLatin1Char('#'),
 #endif
 	};
 
@@ -171,37 +144,37 @@ FileSharingController::FileSharingController(QXmppClient *client)
 		auto reqMan = client->findExtension<QXmppUploadRequestManager>();
 		Q_ASSERT(reqMan);
 
-		connect(reqMan, &QXmppUploadRequestManager::serviceFoundChanged, reqMan, [reqMan]() {
+		connect(reqMan, &QXmppUploadRequestManager::serviceFoundChanged, Kaidan::instance(), [client, reqMan]() {
 			bool supported = reqMan->serviceFound();
 			Kaidan::instance()->serverFeaturesCache()->setHttpUploadSupported(supported);
+
+			if (client->isConnected()) {
+				const auto services = reqMan->uploadServices();
+				const auto limit = services.isEmpty() ? 0 : services.constFirst().sizeLimit();
+
+				AccountDb::instance()->updateAccount(client->configuration().jidBare(), [limit](Account &account) {
+					account.httpUploadLimit = limit;
+				});
+
+				Kaidan::instance()->serverFeaturesCache()->setHttpUploadLimit(limit);
+			}
 		});
 	});
 }
 
-void FileSharingController::sendMessage(Message &&message, bool encrypt)
+auto FileSharingController::sendFiles(QVector<File> files, bool encrypt)
+	-> QXmppTask<SendFilesResult>
 {
-	Q_ASSERT(!message.files.empty());
+	Q_ASSERT(!files.empty());
 
-	message.id = QXmppUtils::generateStanzaUuid();
-	message.stamp = QDateTime::currentDateTimeUtc();
-	message.deliveryState = DeliveryState::Pending;
+	QXmppPromise<SendFilesResult> promise;
+	auto task = promise.task();
 
-	if (!message.fileGroupId) {
-		message.fileGroupId = generateFileId();
-	}
-	for (auto &file : message.files) {
-		file.fileGroupId = *message.fileGroupId;
-		file.id = generateFileId();
-		file.name = QUrl::fromLocalFile(file.localFilePath).fileName();
-	}
-
-	MessageDb::instance()->addMessage(message, MessageOrigin::UserInput);
-
-	auto futures = transform(message.files, [&](auto &file) {
+	auto futures = transform(files, [&](auto &file) {
 		return sendFile(file, encrypt);
 	});
 
-	await(join(this, std::move(futures)), this, [message = std::move(message)](auto &&uploadResults) mutable {
+	await(join(this, std::move(futures)), this, [promise = std::move(promise), files = std::move(files)](auto &&uploadResults) mutable {
 		// Check if any of the uploads failed
 		bool failed = std::any_of(uploadResults.begin(), uploadResults.end(), [](const auto &result) {
 			auto &[id, uploadResult] = result;
@@ -216,10 +189,8 @@ void FileSharingController::sendMessage(Message &&message, bool encrypt)
 			});
 			Q_ASSERT(errorIt != uploadResults.end());
 
-			auto errorText = std::get<QXmppError>(std::get<1>(*errorIt)).description;
-			MessageDb::instance()->updateMessage(message.id, [errorText](auto &message) {
-				message.errorText = tr("Upload failed: %1").arg(errorText);
-			});
+			// currently this only gives the error of the first failed upload
+			promise.finish(std::get<QXmppError>(std::get<1>(std::move(*errorIt))));
 			return;
 		}
 
@@ -229,7 +200,7 @@ void FileSharingController::sendMessage(Message &&message, bool encrypt)
 			return std::pair { fileId, std::get<QXmppFileUpload::FileResult>(uploadResult) };
 		});
 
-		for (auto &file : message.files) {
+		for (auto &file : files) {
 			auto fileResult = fileResultsMap.find(file.id);
 			if (fileResult == fileResultsMap.end()) {
 				continue;
@@ -251,7 +222,7 @@ void FileSharingController::sendMessage(Message &&message, bool encrypt)
 
 				std::optional<qint64> encryptedDataId;
 				if (!s.hashes().empty()) {
-					encryptedDataId = generateFileId();
+					encryptedDataId = MessageDb::instance()->newFileId();
 				}
 
 				return EncryptedSource {
@@ -272,14 +243,10 @@ void FileSharingController::sendMessage(Message &&message, bool encrypt)
 			});
 		}
 
-		MessageDb::instance()->updateMessage(message.id, [files = message.files](auto &message) {
-			message.files = files;
-		});
-
-		runOnThread(Kaidan::instance()->client(), [message = std::move(message)]() mutable {
-			Kaidan::instance()->client()->messageHandler()->sendPendingMessage(std::move(message));
-		});
+		promise.finish(std::move(files));
 	});
+
+	return task;
 }
 
 auto FileSharingController::sendFile(const File &file, bool encrypt)
@@ -335,7 +302,7 @@ void FileSharingController::downloadFile(const QString &messageId, const File &f
 
 	runOnThread(client, [this, client, messageId, fileId = file.id, fileShare = file.toQXmpp()] {
 		QString dirPath = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation) +
-						  QDir::separator() + APPLICATION_DISPLAY_NAME;
+						  QDir::separator() + QStringLiteral(APPLICATION_DISPLAY_NAME);
 
 		if (auto dir = QDir(dirPath); !dir.exists()) {
 			dir.mkpath(QStringLiteral("."));
@@ -364,7 +331,7 @@ void FileSharingController::downloadFile(const QString &messageId, const File &f
 		}();
 
 		auto makeFileName = [&]() -> QString {
-			return dirPath % QDir::separator() % filename % "." % fileExtension;
+			return dirPath % QDir::separator() % filename % QLatin1Char('.') % fileExtension;
 		};
 
 		QString filePath = makeFileName();
@@ -399,7 +366,7 @@ void FileSharingController::downloadFile(const QString &messageId, const File &f
 				auto errorText = std::get<QXmppError>(result).description;
 
 				qDebug() << "[FileSharingController] Couldn't download file:" << errorText;
-				emit Kaidan::instance()->passiveNotificationRequested(
+				Q_EMIT Kaidan::instance()->passiveNotificationRequested(
 					tr("Couldn't download file: %1").arg(errorText));
 			} else if (std::holds_alternative<QXmppFileDownload::Downloaded>(result)) {
 				MessageDb::instance()->updateMessage(messageId, [=](Message &message) {
@@ -434,7 +401,7 @@ void FileSharingController::deleteFile(const QString &messageId, const File &fil
 	});
 
 	// don't delete files not downloaded by us
-	const auto downloadsFolder = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation) + QDir::separator() + APPLICATION_DISPLAY_NAME;
+	const auto downloadsFolder = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation) + QDir::separator() + QStringLiteral(APPLICATION_DISPLAY_NAME);
 	if (file.localFilePath.startsWith(downloadsFolder)) {
 		QFile::remove(file.localFilePath);
 	}
