@@ -1,45 +1,41 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2016 geobra <s.g.b@gmx.de>
+// SPDX-FileCopyrightText: 2016 Marzanna <MRZA-MRZA@users.noreply.github.com>
+// SPDX-FileCopyrightText: 2017 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2018 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2019 Filipe Azevedo <pasnox@gmail.com>
+// SPDX-FileCopyrightText: 2019 Melvin Keskin <melvo@olomono.de>
+// SPDX-FileCopyrightText: 2019 Robert Maerkisch <zatroxde@protonmail.ch>
+// SPDX-FileCopyrightText: 2020 caca hueto <cacahueto@olomono.de>
+// SPDX-FileCopyrightText: 2020 Mathis Brüchert <mbblp@protonmail.ch>
+// SPDX-FileCopyrightText: 2022 Bhavy Airi <airiragahv@gmail.com>
+// SPDX-FileCopyrightText: 2023 Tibor Csötönyi <work@taibsu.de>
+// SPDX-FileCopyrightText: 2024 Filipe Azevedo <pasnox@gmail.com>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #pragma once
 
 // Kaidan
+#include "AccountMigrationManager.h"
 #include "ClientWorker.h"
 
 class QSize;
+class AccountDb;
+class BlockingController;
+class ChatController;
+class ChatHintModel;
+class ClientThread;
 class Database;
 class DataFormModel;
+class EncryptionController;
+class FileSharingController;
+class GroupChatUserDb;
+class GroupChatController;
+class MessageController;
+class MessageDb;
+class MessageModel;
 class Notifications;
 class RosterDb;
-class MessageDb;
-class FileSharingController;
 
 /**
  * @class Kaidan Kaidan's Back-End Class
@@ -55,11 +51,13 @@ class Kaidan : public QObject
 	Q_OBJECT
 
 	Q_PROPERTY(ClientWorker* client READ client CONSTANT)
+	Q_PROPERTY(BlockingController *blockingController READ blockingController CONSTANT)
 	Q_PROPERTY(FileSharingController *fileSharingController READ fileSharingController CONSTANT)
+	Q_PROPERTY(GroupChatController *groupChatController READ groupChatController CONSTANT)
 	Q_PROPERTY(AvatarFileStorage* avatarStorage READ avatarStorage NOTIFY avatarStorageChanged)
 	Q_PROPERTY(ServerFeaturesCache* serverFeaturesCache READ serverFeaturesCache CONSTANT)
 	Q_PROPERTY(Settings* settings READ settings CONSTANT)
-	Q_PROPERTY(quint8 connectionState READ connectionState NOTIFY connectionStateChanged)
+	Q_PROPERTY(quint8 connectionState READ connectionStateId NOTIFY connectionStateChanged)
 	Q_PROPERTY(QString connectionStateText READ connectionStateText NOTIFY connectionStateChanged)
 	Q_PROPERTY(quint8 connectionError READ connectionError NOTIFY connectionErrorChanged)
 
@@ -103,15 +101,7 @@ public:
 	 */
 	Q_INVOKABLE void logIn()
 	{
-		emit logInRequested();
-	}
-
-	/**
-	 * Connects to the server and requests a data form for account registration.
-	 */
-	Q_INVOKABLE void requestRegistrationForm()
-	{
-		emit registrationFormRequested();
+		Q_EMIT logInRequested();
 	}
 
 	/**
@@ -122,18 +112,16 @@ public:
 	 */
 	Q_INVOKABLE void logOut()
 	{
-		emit logOutRequested();
+		Q_EMIT logOutRequested();
 	}
 
 	/**
 	 * Returns the current ConnectionState
 	 */
-	quint8 connectionState() const
-	{
-		return quint8(m_connectionState);
-	}
-
+	quint8 connectionStateId() const { return quint8(m_connectionState); }
+	Enums::ConnectionState connectionState() const { return m_connectionState; }
 	QString connectionStateText() const;
+	bool connected() const { return connectionState() == Enums::ConnectionState::StateConnected; }
 
 	/**
 	 * Returns the last connection error.
@@ -141,17 +129,20 @@ public:
 	quint8 connectionError() const { return quint8(m_connectionError); }
 
 	ClientWorker *client() const { return m_client; }
+	BlockingController *blockingController() const { return m_blockingController.get(); }
 	FileSharingController *fileSharingController() const { return m_fileSharingController.get(); }
+	GroupChatController *groupChatController() const { return m_groupChatController; }
 	AvatarFileStorage *avatarStorage() const { return m_caches->avatarStorage; }
 	ServerFeaturesCache *serverFeaturesCache() const { return m_caches->serverFeaturesCache; }
 	VCardCache *vCardCache() const { return m_caches->vCardCache; }
+
 	Settings *settings() const { return m_caches->settings; }
 	Database *database() const { return m_database; }
 
 	/**
 	 * Adds XMPP URI to open as soon as possible
 	 */
-	void addOpenUri(const QString &uri);
+	void addOpenUri(const QString &uriString);
 
 	/**
 	 * Connects to the server by the parsed credentials (bare JID and
@@ -173,11 +164,11 @@ public:
 	 *
 	 * In all other cases, LoginByUriState::InvalidLoginUri is returned.
 	 *
-	 * @param uri string which can be an XMPP login URI
+	 * @param uriString string which can be an XMPP login URI
 	 *
 	 * @return the state which specifies how the XMPP login URI was used
 	 */
-	Q_INVOKABLE quint8 logInByUri(const QString &uri);
+	Q_INVOKABLE quint8 logInByUri(const QString &uriString);
 
 	/**
 	 * Authenticates or distrusts end-to-end encryption keys by a given XMPP URI
@@ -186,19 +177,28 @@ public:
 	 * Only if the URI's JID matches the expectedJid or no expectedJid is
 	 * passed, the trust decision is made.
 	 *
-	 * @param uri string which can be an XMPP Trust Message URI
+	 * @param uriString string which can be an XMPP Trust Message URI
 	 * @param expectedJid JID of the key owner whose keys are expected to be
 	 *        authenticated or none to allow all JIDs
 	 */
-	Q_INVOKABLE Kaidan::TrustDecisionByUriResult makeTrustDecisionsByUri(const QString &uri, const QString &expectedJid = {});
+	Q_INVOKABLE Kaidan::TrustDecisionByUriResult makeTrustDecisionsByUri(const QString &uriString, const QString &expectedJid = {});
+
+	Q_SLOT void startAccountMigration();
+	Q_SLOT void continueAccountMigration(const QVariant &userData = {});
+	Q_SLOT void cancelAccountMigration();
+	Q_SIGNAL void accountMigrationStateChanged(AccountMigrationManager::MigrationState state);
+
+	Q_INVOKABLE bool testAccountMigrationState(AccountMigrationManager::MigrationState state);
+
+	Q_SIGNAL void accountErrorOccurred(const QString &msg);
+	Q_SIGNAL void accountBusyChanged(bool busy);
 
-signals:
 	/**
 	 * Emitted when a data form for registration is received from the server.
 	 *
 	 * @param dataFormModel received model for the registration data form
 	 */
-	void registrationFormReceived(DataFormModel *dataFormModel);
+	Q_SIGNAL void registrationFormReceived(DataFormModel *dataFormModel);
 
 	/**
 	 * Emitted when an out-of-band URL for registration is received from the
@@ -206,13 +206,7 @@ signals:
 	 *
 	 * @param outOfBandUrl URL used for out-of-band registration
 	 */
-	void registrationOutOfBandUrlReceived(const QString &outOfBandUrl);
-
-	/**
-	 * Emitted to request a registration form from the server which is set as the
-	 * currently used JID.
-	 */
-	void registrationFormRequested();
+	Q_SIGNAL void registrationOutOfBandUrlReceived(const QUrl &outOfBandUrl);
 
 	/**
 	 * Emitted when the account registration failed.
@@ -220,58 +214,63 @@ signals:
 	 * @param error received error
 	 * @param errorMessage message describing the error
 	 */
-	void registrationFailed(quint8 error, const QString &errorMessage);
+	Q_SIGNAL void registrationFailed(quint8 error, const QString &errorMessage);
 
 	/**
 	 * Emitted to log in to the server with the set credentials.
 	 */
-	void logInRequested();
+	Q_SIGNAL void logInRequested();
 
 	/**
 	 * Emitted to log out of the server.
 	 *
 	 * @param isApplicationBeingClosed true if the application will be terminated directly after logging out, false otherwise
 	 */
-	void logOutRequested(bool isApplicationBeingClosed = false);
+	Q_SIGNAL void logOutRequested(bool isApplicationBeingClosed = false);
 
-	void avatarStorageChanged();
+	Q_SIGNAL void avatarStorageChanged();
 
 	/**
 	 * Emitted, when the client's connection state has changed (e.g. when
 	 * successfully connected or when disconnected)
 	 */
-	void connectionStateChanged();
+	Q_SIGNAL void connectionStateChanged();
 
 	/**
 	 * Emitted when the connection error changed.
 	 *
 	 * That is the case when the client failed to connect or it succeeded to connect after an error.
 	 */
-	void connectionErrorChanged();
+	Q_SIGNAL void connectionErrorChanged();
 
 	/**
 	 * Emitted when there are no (correct) credentials and new ones are needed.
 	 *
 	 * The client will be in disconnected state when this is emitted.
 	 */
-	void credentialsNeeded();
+	Q_SIGNAL void credentialsNeeded();
 
 	/**
 	 * Emitted when an authenticated connection to the server is established with new credentials for the first time.
 	 *
 	 * The client will be in connected state when this is emitted.
 	 */
-	void loggedInWithNewCredentials();
+	Q_SIGNAL void loggedInWithNewCredentials();
 
 	/**
 	 * Raises the window to the foreground so that it is on top of all other windows.
 	 */
-	void raiseWindowRequested();
+	Q_SIGNAL void raiseWindowRequested();
+
+	/**
+	 * Opens the start page on top of any existing pages on the lowest layer.
+	 */
+	Q_SIGNAL void openStartPageRequested();
 
 	/**
 	 * Opens the view with the roster and empty chat page.
 	 */
-	void openChatViewRequested();
+	Q_SIGNAL void openChatViewRequested();
 
 	/**
 	 * Opens the chat page for a given chat.
@@ -279,99 +278,89 @@ signals:
 	 * @param accountJid JID of the account for that the chat page is opened
 	 * @param chatJid JID of the chat for that the chat page is opened
 	 */
-	void openChatPageRequested(const QString &accountJid, const QString &chatJid);
+	Q_SIGNAL void openChatPageRequested(const QString &accountJid, const QString &chatJid);
+
+	/**
+	 * Closes the chat page.
+	 */
+	Q_SIGNAL void closeChatPageRequested();
 
 	/**
 	 * Emitted when the removal state of the password on the account transfer page changed.
 	 */
-	void passwordVisibilityChanged();
+	Q_SIGNAL void passwordVisibilityChanged();
 
 	/**
 	 * Show passive notification
 	 */
-	void passiveNotificationRequested(QString text);
+	Q_SIGNAL void passiveNotificationRequested(QString text, const QString duration = {});
 
 	/**
 	 * XMPP URI received
 	 *
 	 * Is called when Kaidan was used to open an XMPP URI (i.e. 'xmpp:kaidan@muc.kaidan.im?join')
 	 */
-	void xmppUriReceived(QString uri);
+	Q_SIGNAL void xmppUriReceived(QString uriString);
 
 	/**
 	 * Emitted when changing of the user's password finished succfessully.
 	 */
-	void passwordChangeSucceeded();
+	Q_SIGNAL void passwordChangeSucceeded();
 
 	/**
 	 * Emitted when changing the user's password failed.
 	 *
 	 * @param errorMessage message describing the error
 	 */
-	void passwordChangeFailed(const QString &errorMessage);
+	Q_SIGNAL void passwordChangeFailed(const QString &errorMessage);
 
 	/**
 	 * Emitted when changing of the user's avatar finished succfessully.
 	 */
-	void avatarChangeSucceeded();
+	Q_SIGNAL void avatarChangeSucceeded();
 
-	/**
-	 * Emitted, when a contact was muted/unmuted.
-	 */
-	void notificationsMutedChanged(const QString& jid);
-
-	/**
-	 * Deletes the account data from the client and server.
-	 */
-	void deleteAccountFromClientAndServer();
-
-	/**
-	 * Deletes the account data from the configuration file and database.
-	 */
-	void deleteAccountFromClient();
-
-public slots:
 	/**
 	 * Set current connection state
 	 */
-	void setConnectionState(Enums::ConnectionState connectionState);
+	Q_INVOKABLE void setConnectionState(Enums::ConnectionState connectionState);
 
 	/**
 	 * Sets a new connection error.
 	 */
-	void setConnectionError(ClientWorker::ConnectionError error);
+	Q_INVOKABLE void setConnectionError(ClientWorker::ConnectionError error);
 
 	/**
 	 * Receives messages from another instance of the application
 	 */
-	void receiveMessage(quint32, const QByteArray &msg)
+	Q_INVOKABLE void receiveMessage(quint32, const QByteArray &msg)
 	{
 		// currently we only send XMPP URIs
-		addOpenUri(msg);
+		addOpenUri(QString::fromUtf8(msg));
 	}
 
-	/**
-	 * Returns whether notifications are enabled for the given contact.
-	 */
-	bool notificationsMuted(const QString& jid);
-
-	/**
-	 * Sets the notifications to muted/unmuted.
-	 *
-	 * @param muted true if notifications should be muted
-	 * @param jid JID of the entity for that notifications should be muted or unmuted
-	 */
-	void setNotificationsMuted(const QString &jid, bool muted);
-
 private:
+	void initializeAccountMigration();
+
 	Notifications *m_notifications;
+
 	Database *m_database;
+	AccountDb *m_accountDb;
 	MessageDb *m_msgDb;
 	RosterDb *m_rosterDb;
-	QThread *m_cltThrd;
+	GroupChatUserDb *m_groupChatUserDb;
+
+	ClientThread *m_cltThrd;
 	ClientWorker::Caches *m_caches;
 	ClientWorker *m_client;
+	std::unique_ptr<BlockingController> m_blockingController;
+	ChatController *m_chatController;
+	EncryptionController *m_encryptionController;
 	std::unique_ptr<FileSharingController> m_fileSharingController;
+	GroupChatController *m_groupChatController;
+	MessageController *m_messageController;
+
+	MessageModel *m_messageModel;
+	ChatHintModel *m_chatHintModel;
 
 	QString m_openUriCache;
 	Enums::ConnectionState m_connectionState = Enums::ConnectionState::StateDisconnected;
