@@ -1,32 +1,7 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2020 Linus Jahn <lnj@kaidan.im>
+// SPDX-FileCopyrightText: 2020 Melvin Keskin <melvo@olomono.de>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "RegistrationDataFormModel.h"
 // Qt
@@ -95,7 +70,7 @@ int RegistrationDataFormModel::usernameFieldIndex() const
 	const QList<QXmppDataForm::Field> &fields = m_form.fields();
 	for (int i = 0; i < fields.size(); i++) {
 		if (fields.at(i).type() == QXmppDataForm::Field::TextSingleField &&
-				fields.at(i).key().compare(USERNAME, Qt::CaseInsensitive) == 0) {
+				fields.at(i).key().compare(QStringLiteral(USERNAME), Qt::CaseInsensitive) == 0) {
 			return i;
 		}
 	}
@@ -107,7 +82,7 @@ int RegistrationDataFormModel::passwordFieldIndex() const
 	const QList<QXmppDataForm::Field> &fields = m_form.fields();
 	for (int i = 0; i < fields.size(); i++) {
 		if (fields.at(i).type() == QXmppDataForm::Field::TextPrivateField &&
-				fields.at(i).key().compare(PASSWORD, Qt::CaseInsensitive) == 0) {
+				fields.at(i).key().compare(QStringLiteral(PASSWORD), Qt::CaseInsensitive) == 0) {
 			return i;
 		}
 	}
@@ -119,7 +94,7 @@ int RegistrationDataFormModel::emailFieldIndex() const
 	const QList<QXmppDataForm::Field> &fields = m_form.fields();
 	for (int i = 0; i < fields.size(); i++) {
 		if (fields.at(i).type() == QXmppDataForm::Field::TextSingleField &&
-				fields.at(i).key().compare(EMAIL, Qt::CaseInsensitive) == 0) {
+				fields.at(i).key().compare(QStringLiteral(EMAIL), Qt::CaseInsensitive) == 0) {
 			return i;
 		}
 	}
@@ -175,9 +150,10 @@ QVector<int> RegistrationDataFormModel::indiciesToFilter() const
 
 	// username and password
 	// email is currently not filtered because we do not have an extra email view
-	for (const auto &index : {usernameFieldIndex(), passwordFieldIndex()}) {
-		if (index != -1)
+	for (const auto &index : { usernameFieldIndex(), passwordFieldIndex() }) {
+		if (index != -1) {
 			indicies << index;
+		}
 	}
 
 	// search for other common fields to filter for
@@ -193,12 +169,12 @@ QVector<int> RegistrationDataFormModel::indiciesToFilter() const
 void RegistrationDataFormModel::initializeFilteredDataFormFields()
 {
 	m_filteredDataFormFields = {
-		FORM_TYPE,
-		FROM,
-		CAPTCHA_FALLBACK_TEXT,
-		CAPTCHA_FALLBACK_URL,
-		CAPTCHAHIDDEN,
-		CHALLENGE,
-		SID
+		QStringLiteral(FORM_TYPE),
+		QStringLiteral(FROM),
+		QStringLiteral(CAPTCHA_FALLBACK_TEXT),
+		QStringLiteral(CAPTCHA_FALLBACK_URL),
+		QStringLiteral(CAPTCHAHIDDEN),
+		QStringLiteral(CHALLENGE),
+		QStringLiteral(SID)
 	};
 }
