@@ -1,3 +1,7 @@
+// SPDX-FileCopyrightText: 2016 J-P Nurmi
+//
+// SPDX-License-Identifier: MIT
+
 /*
  * Copyright (c) 2016 J-P Nurmi
  *
@@ -42,7 +46,7 @@ QColor StatusBar::color() const
 void StatusBar::setColor(const QColor &color)
 {
 	m_color = color;
-	emit colorChanged();
+	Q_EMIT colorChanged();
 	if (!isAvailable())
 		return;
 
