@@ -1,32 +1,7 @@
-/*
- *  Kaidan - A user-friendly XMPP client for every device!
- *
- *  Copyright (C) 2016-2023 Kaidan developers and contributors
- *  (see the LICENSE file for a full list of copyright authors)
- *
- *  Kaidan is free software: you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation, either version 3 of the License, or
- *  (at your option) any later version.
- *
- *  In addition, as a special exception, the author of Kaidan gives
- *  permission to link the code of its release with the OpenSSL
- *  project's "OpenSSL" library (or with modified versions of it that
- *  use the same license as the "OpenSSL" library), and distribute the
- *  linked executables. You must obey the GNU General Public License in
- *  all respects for all of the code used other than "OpenSSL". If you
- *  modify this file, you may extend this exception to your version of
- *  the file, but you are not obligated to do so.  If you do not wish to
- *  do so, delete this exception statement from your version.
- *
- *  Kaidan is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with Kaidan.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2020 Jonah Brüchert <jbb@kaidan.im>
+// SPDX-FileCopyrightText: 2021 Linus Jahn <lnj@kaidan.im>
+//
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "UserDevicesModel.h"
 
@@ -110,9 +85,9 @@ void UserDevicesModel::setJid(const QString &jid)
 	endResetModel();
 
 	// request version data for all available resources
-	emit clientVersionsRequested(m_jid);
+	Q_EMIT clientVersionsRequested(m_jid);
 
-	emit jidChanged();
+	Q_EMIT jidChanged();
 }
 
 void UserDevicesModel::handleClientVersionReceived(const QXmppVersionIq &versionIq)
@@ -132,7 +107,7 @@ void UserDevicesModel::handleClientVersionReceived(const QXmppVersionIq &version
 		info->os = versionIq.os();
 
 		const int i = std::distance(m_devices.begin(), info);
-		emit dataChanged(index(i), index(i), { Name, Version, OS });
+		Q_EMIT dataChanged(index(i), index(i), { Name, Version, OS });
 	}
 }
 
@@ -147,7 +122,7 @@ void UserDevicesModel::handlePresenceChanged(PresenceCache::ChangeType type, con
 		m_devices.append(DeviceInfo(resource));
 		endInsertRows();
 
-		emit clientVersionsRequested(m_jid, resource);
+		Q_EMIT clientVersionsRequested(m_jid, resource);
 		break;
 	case PresenceCache::Disconnected:
 		for (int i = 0; i < m_devices.count(); i++) {
