@@ -1,4 +1,7 @@
 #!/bin/bash
+# SPDX-FileCopyrightText: 2019 Filipe Azevedo <pasnox@gmail.com>
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 SCRIPT_DIR=$(dirname "${0}")
 
@@ -17,20 +20,20 @@ set -e
 # find "${SOURCE_DIR}" -iname '*.qml' -exec grep 'import ' {} \; | sort -u
 
 declare -A QT
-QT["QtGraphicalEffects"]="1.14"
-QT["Qt.labs.folderlistmodel"]="2.1"
-QT["QtLocation"]="5.14"
-QT["QtMultimedia"]="5.14"
-QT["QtPositioning"]="5.14"
-QT["QtQml"]="2.14"
-QT["QtQuick"]="2.14"
-QT["QtQuick.Controls"]="2.14"
-QT["QtQuick.Controls.Material"]="2.14"
+QT["QtGraphicalEffects"]="1.15"
+QT["QtLocation"]="5.15"
+QT["QtMultimedia"]="5.15"
+QT["QtPositioning"]="5.15"
+QT["QtQuick"]="2.15"
+QT["QtQuick.Controls"]="2.15"
+QT["QtQuick.Controls.Material"]="2.15"
 QT["QtQuick.Dialogs"]="1.3"
-QT["QtQuick.Layouts"]="1.14"
+QT["QtQuick.Layouts"]="1.15"
 
 declare -A KIRIGAMI
 KIRIGAMI["org.kde.kirigami"]="2.19"
+KIRIGAMI["org.kde.kirigamiaddons.labs.mobileform"]="0.1"
+KIRIGAMI["org.kde.kirigamiaddons.labs.components"]="1.0"
 
 declare -A KAIDAN
 KAIDAN["EmojiModel"]="0.1"
